@@ -30,6 +30,9 @@
 
 package com.google.protobuf;
 
+import static com.google.common.truth.Truth.assertThat;
+import static com.google.common.truth.Truth.assertWithMessage;
+
 import com.google.protobuf.CodedOutputStream.OutOfSpaceException;
 import protobuf_unittest.UnittestProto.SparseEnumMessage;
 import protobuf_unittest.UnittestProto.TestAllTypes;
@@ -41,14 +44,13 @@ import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
-import junit.framework.TestCase;
+import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.JUnit4;
 
-/**
- * Unit test for {@link CodedOutputStream}.
- *
- * @author kenton@google.com Kenton Varda
- */
-public class CodedOutputStreamTest extends TestCase {
+/** Unit test for {@link CodedOutputStream}. */
+@RunWith(JUnit4.class)
+public class CodedOutputStreamTest {
   private interface Coder {
     CodedOutputStream stream();
 
@@ -224,6 +226,7 @@ public class CodedOutputStreamTest extends TestCase {
   }
 
   /** Checks that invariants are maintained for varint round trip input and output. */
+  @Test
   public void testVarintRoundTrips() throws Exception {
     for (OutputType outputType : OutputType.values()) {
       assertVarintRoundTrip(outputType, 0L);
@@ -238,6 +241,7 @@ public class CodedOutputStreamTest extends TestCase {
   }
 
   /** Tests writeRawVarint32() and writeRawVarint64(). */
+  @Test
   public void testWriteVarint() throws Exception {
     assertWriteVarint(bytes(0x00), 0);
     assertWriteVarint(bytes(0x01), 1);
@@ -281,6 +285,7 @@ public class CodedOutputStreamTest extends TestCase {
   }
 
   /** Tests writeRawLittleEndian32() and writeRawLittleEndian64(). */
+  @Test
   public void testWriteLittleEndian() throws Exception {
     assertWriteLittleEndian32(bytes(0x78, 0x56, 0x34, 0x12), 0x12345678);
     assertWriteLittleEndian32(bytes(0xf0, 0xde, 0xbc, 0x9a), 0x9abcdef0);
@@ -292,50 +297,61 @@ public class CodedOutputStreamTest extends TestCase {
   }
 
   /** Test encodeZigZag32() and encodeZigZag64(). */
+  @Test
   public void testEncodeZigZag() throws Exception {
-    assertEquals(0, CodedOutputStream.encodeZigZag32(0));
-    assertEquals(1, CodedOutputStream.encodeZigZag32(-1));
-    assertEquals(2, CodedOutputStream.encodeZigZag32(1));
-    assertEquals(3, CodedOutputStream.encodeZigZag32(-2));
-    assertEquals(0x7FFFFFFE, CodedOutputStream.encodeZigZag32(0x3FFFFFFF));
-    assertEquals(0x7FFFFFFF, CodedOutputStream.encodeZigZag32(0xC0000000));
-    assertEquals(0xFFFFFFFE, CodedOutputStream.encodeZigZag32(0x7FFFFFFF));
-    assertEquals(0xFFFFFFFF, CodedOutputStream.encodeZigZag32(0x80000000));
+    assertThat(CodedOutputStream.encodeZigZag32(0)).isEqualTo(0);
+    assertThat(CodedOutputStream.encodeZigZag32(-1)).isEqualTo(1);
+    assertThat(CodedOutputStream.encodeZigZag32(1)).isEqualTo(2);
+    assertThat(CodedOutputStream.encodeZigZag32(-2)).isEqualTo(3);
+    assertThat(CodedOutputStream.encodeZigZag32(0x3FFFFFFF)).isEqualTo(0x7FFFFFFE);
+    assertThat(CodedOutputStream.encodeZigZag32(0xC0000000)).isEqualTo(0x7FFFFFFF);
+    assertThat(CodedOutputStream.encodeZigZag32(0x7FFFFFFF)).isEqualTo(0xFFFFFFFE);
+    assertThat(CodedOutputStream.encodeZigZag32(0x80000000)).isEqualTo(0xFFFFFFFF);
 
-    assertEquals(0, CodedOutputStream.encodeZigZag64(0));
-    assertEquals(1, CodedOutputStream.encodeZigZag64(-1));
-    assertEquals(2, CodedOutputStream.encodeZigZag64(1));
-    assertEquals(3, CodedOutputStream.encodeZigZag64(-2));
-    assertEquals(0x000000007FFFFFFEL, CodedOutputStream.encodeZigZag64(0x000000003FFFFFFFL));
-    assertEquals(0x000000007FFFFFFFL, CodedOutputStream.encodeZigZag64(0xFFFFFFFFC0000000L));
-    assertEquals(0x00000000FFFFFFFEL, CodedOutputStream.encodeZigZag64(0x000000007FFFFFFFL));
-    assertEquals(0x00000000FFFFFFFFL, CodedOutputStream.encodeZigZag64(0xFFFFFFFF80000000L));
-    assertEquals(0xFFFFFFFFFFFFFFFEL, CodedOutputStream.encodeZigZag64(0x7FFFFFFFFFFFFFFFL));
-    assertEquals(0xFFFFFFFFFFFFFFFFL, CodedOutputStream.encodeZigZag64(0x8000000000000000L));
+    assertThat(CodedOutputStream.encodeZigZag64(0)).isEqualTo(0);
+    assertThat(CodedOutputStream.encodeZigZag64(-1)).isEqualTo(1);
+    assertThat(CodedOutputStream.encodeZigZag64(1)).isEqualTo(2);
+    assertThat(CodedOutputStream.encodeZigZag64(-2)).isEqualTo(3);
+    assertThat(CodedOutputStream.encodeZigZag64(0x000000003FFFFFFFL))
+        .isEqualTo(0x000000007FFFFFFEL);
+    assertThat(CodedOutputStream.encodeZigZag64(0xFFFFFFFFC0000000L))
+        .isEqualTo(0x000000007FFFFFFFL);
+    assertThat(CodedOutputStream.encodeZigZag64(0x000000007FFFFFFFL))
+        .isEqualTo(0x00000000FFFFFFFEL);
+    assertThat(CodedOutputStream.encodeZigZag64(0xFFFFFFFF80000000L))
+        .isEqualTo(0x00000000FFFFFFFFL);
+    assertThat(CodedOutputStream.encodeZigZag64(0x7FFFFFFFFFFFFFFFL))
+        .isEqualTo(0xFFFFFFFFFFFFFFFEL);
+    assertThat(CodedOutputStream.encodeZigZag64(0x8000000000000000L))
+        .isEqualTo(0xFFFFFFFFFFFFFFFFL);
 
     // Some easier-to-verify round-trip tests.  The inputs (other than 0, 1, -1)
     // were chosen semi-randomly via keyboard bashing.
-    assertEquals(0, CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(0)));
-    assertEquals(1, CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(1)));
-    assertEquals(-1, CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(-1)));
-    assertEquals(14927, CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(14927)));
-    assertEquals(-3612, CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(-3612)));
+    assertThat(CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(0))).isEqualTo(0);
+    assertThat(CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(1))).isEqualTo(1);
+    assertThat(CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(-1))).isEqualTo(-1);
+    assertThat(CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(14927)))
+        .isEqualTo(14927);
+    assertThat(CodedOutputStream.encodeZigZag32(CodedInputStream.decodeZigZag32(-3612)))
+        .isEqualTo(-3612);
 
-    assertEquals(0, CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(0)));
-    assertEquals(1, CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(1)));
-    assertEquals(-1, CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(-1)));
-    assertEquals(14927, CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(14927)));
-    assertEquals(-3612, CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(-3612)));
+    assertThat(CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(0))).isEqualTo(0);
+    assertThat(CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(1))).isEqualTo(1);
+    assertThat(CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(-1))).isEqualTo(-1);
+    assertThat(CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(14927)))
+        .isEqualTo(14927);
+    assertThat(CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(-3612)))
+        .isEqualTo(-3612);
 
-    assertEquals(
-        856912304801416L,
-        CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(856912304801416L)));
-    assertEquals(
-        -75123905439571256L,
-        CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(-75123905439571256L)));
+    assertThat(CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(856912304801416L)))
+        .isEqualTo(856912304801416L);
+    assertThat(
+            CodedOutputStream.encodeZigZag64(CodedInputStream.decodeZigZag64(-75123905439571256L)))
+        .isEqualTo(-75123905439571256L);
   }
 
   /** Tests writing a whole message with every field type. */
+  @Test
   public void testWriteWholeMessage() throws Exception {
     final byte[] expectedBytes = TestUtil.getGoldenMessage().toByteArray();
     TestAllTypes message = TestUtil.getAllSet();
@@ -361,6 +377,7 @@ public class CodedOutputStreamTest extends TestCase {
    * Tests writing a whole message with every packed field type. Ensures the wire format of packed
    * fields is compatible with C++.
    */
+  @Test
   public void testWriteWholePackedFieldsMessage() throws Exception {
     byte[] expectedBytes = TestUtil.getGoldenPackedFieldsMessage().toByteArray();
     TestPackedTypes message = TestUtil.getPackedSet();
@@ -378,21 +395,23 @@ public class CodedOutputStreamTest extends TestCase {
    * Test writing a message containing a negative enum value. This used to fail because the size was
    * not properly computed as a sign-extended varint.
    */
+  @Test
   public void testWriteMessageWithNegativeEnumValue() throws Exception {
     SparseEnumMessage message =
         SparseEnumMessage.newBuilder().setSparseEnum(TestSparseEnum.SPARSE_E).build();
-    assertTrue(message.getSparseEnum().getNumber() < 0);
+    assertThat(message.getSparseEnum().getNumber()).isLessThan(0);
     for (OutputType outputType : OutputType.values()) {
       Coder coder = outputType.newCoder(message.getSerializedSize());
       message.writeTo(coder.stream());
       coder.stream().flush();
       byte[] rawBytes = coder.toByteArray();
       SparseEnumMessage message2 = SparseEnumMessage.parseFrom(rawBytes);
-      assertEquals(TestSparseEnum.SPARSE_E, message2.getSparseEnum());
+      assertThat(message2.getSparseEnum()).isEqualTo(TestSparseEnum.SPARSE_E);
     }
   }
 
   /** Test getTotalBytesWritten() */
+  @Test
   public void testGetTotalBytesWritten() throws Exception {
     Coder coder = OutputType.STREAM.newCoder(4 * 1024);
 
@@ -402,26 +421,27 @@ public class CodedOutputStreamTest extends TestCase {
     for (int i = 0; i < 1024; ++i) {
       coder.stream().writeRawBytes(value, 0, value.length);
     }
-    assertEquals(value.length * 1024, coder.stream().getTotalBytesWritten());
+    assertThat(coder.stream().getTotalBytesWritten()).isEqualTo(value.length * 1024);
 
     // Now write an encoded string.
     String string =
         "abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz";
     // Ensure we take the slower fast path.
-    assertTrue(
-        CodedOutputStream.computeUInt32SizeNoTag(string.length())
-            != CodedOutputStream.computeUInt32SizeNoTag(string.length() * Utf8.MAX_BYTES_PER_CHAR));
+    assertThat(CodedOutputStream.computeUInt32SizeNoTag(string.length()))
+        .isNotEqualTo(
+            CodedOutputStream.computeUInt32SizeNoTag(string.length() * Utf8.MAX_BYTES_PER_CHAR));
 
     coder.stream().writeStringNoTag(string);
     coder.stream().flush();
     int stringSize = CodedOutputStream.computeStringSizeNoTag(string);
 
     // Verify that the total bytes written is correct
-    assertEquals((value.length * 1024) + stringSize, coder.stream().getTotalBytesWritten());
+    assertThat(coder.stream().getTotalBytesWritten()).isEqualTo((value.length * 1024) + stringSize);
   }
 
   // TODO(dweis): Write a comprehensive test suite for CodedOutputStream that covers more than just
   //    this case.
+  @Test
   public void testWriteStringNoTag_fastpath() throws Exception {
     int bufferSize = 153;
     String threeBytesPer = "\u0981";
@@ -430,10 +450,10 @@ public class CodedOutputStreamTest extends TestCase {
       string += threeBytesPer;
     }
     // These checks ensure we will tickle the slower fast path.
-    assertEquals(1, CodedOutputStream.computeUInt32SizeNoTag(string.length()));
-    assertEquals(
-        2, CodedOutputStream.computeUInt32SizeNoTag(string.length() * Utf8.MAX_BYTES_PER_CHAR));
-    assertEquals(bufferSize, string.length() * Utf8.MAX_BYTES_PER_CHAR);
+    assertThat(CodedOutputStream.computeUInt32SizeNoTag(string.length())).isEqualTo(1);
+    assertThat(CodedOutputStream.computeUInt32SizeNoTag(string.length() * Utf8.MAX_BYTES_PER_CHAR))
+        .isEqualTo(2);
+    assertThat(bufferSize).isEqualTo(string.length() * Utf8.MAX_BYTES_PER_CHAR);
 
     for (OutputType outputType : OutputType.values()) {
       Coder coder = outputType.newCoder(bufferSize + 2);
@@ -442,6 +462,7 @@ public class CodedOutputStreamTest extends TestCase {
     }
   }
 
+  @Test
   public void testWriteToByteBuffer() throws Exception {
     final int bufferSize = 16 * 1024;
     ByteBuffer buffer = ByteBuffer.allocate(bufferSize);
@@ -462,19 +483,20 @@ public class CodedOutputStreamTest extends TestCase {
     codedStream.flush();
 
     // Check that data is correctly written to the ByteBuffer.
-    assertEquals(0, buffer.remaining());
+    assertThat(buffer.remaining()).isEqualTo(0);
     buffer.flip();
     for (int i = 0; i < length1; i++) {
-      assertEquals((byte) 1, buffer.get());
+      assertThat(buffer.get()).isEqualTo((byte) 1);
     }
     for (int i = 0; i < length2; i++) {
-      assertEquals((byte) 2, buffer.get());
+      assertThat(buffer.get()).isEqualTo((byte) 2);
     }
     for (int i = 0; i < length3; i++) {
-      assertEquals((byte) 3, buffer.get());
+      assertThat(buffer.get()).isEqualTo((byte) 3);
     }
   }
 
+  @Test
   public void testWriteByteBuffer() throws Exception {
     byte[] value = "abcde".getBytes(Internal.UTF_8);
     ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
@@ -484,31 +506,33 @@ public class CodedOutputStreamTest extends TestCase {
     // ByteBuffer's capacity() is 5.
     codedStream.writeRawBytes(byteBuffer);
     // The above call shouldn't affect the ByteBuffer's state.
-    assertEquals(0, byteBuffer.position());
-    assertEquals(1, byteBuffer.limit());
+    assertThat(byteBuffer.position()).isEqualTo(0);
+    assertThat(byteBuffer.limit()).isEqualTo(1);
 
     // The correct way to write part of an array using ByteBuffer.
     codedStream.writeRawBytes(ByteBuffer.wrap(value, 2, 1).slice());
 
     codedStream.flush();
     byte[] result = outputStream.toByteArray();
-    assertEquals(6, result.length);
+    assertThat(result).hasLength(6);
     for (int i = 0; i < 5; i++) {
-      assertEquals(value[i], result[i]);
+      assertThat(value[i]).isEqualTo(result[i]);
     }
-    assertEquals(value[2], result[5]);
+    assertThat(value[2]).isEqualTo(result[5]);
   }
 
+  @Test
   public void testWriteByteArrayWithOffsets() throws Exception {
     byte[] fullArray = bytes(0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88);
     for (OutputType type : new OutputType[] {OutputType.ARRAY}) {
       Coder coder = type.newCoder(4);
       coder.stream().writeByteArrayNoTag(fullArray, 2, 2);
       assertEqualBytes(type, bytes(0x02, 0x33, 0x44), coder.toByteArray());
-      assertEquals(3, coder.stream().getTotalBytesWritten());
+      assertThat(coder.stream().getTotalBytesWritten()).isEqualTo(3);
     }
   }
 
+  @Test
   public void testSerializeUtf8_MultipleSmallWrites() throws Exception {
     final String source = "abcdefghijklmnopqrstuvwxyz";
 
@@ -533,6 +557,7 @@ public class CodedOutputStreamTest extends TestCase {
     }
   }
 
+  @Test
   public void testSerializeInvalidUtf8() throws Exception {
     String[] invalidStrings =
         new String[] {
@@ -558,6 +583,7 @@ public class CodedOutputStreamTest extends TestCase {
 
   // TODO(nathanmittler): This test can be deleted once we properly throw IOException while
   // encoding invalid UTF-8 strings.
+  @Test
   public void testSerializeInvalidUtf8FollowedByOutOfSpace() throws Exception {
     final int notEnoughBytes = 4;
     CodedOutputStream outputWithArray = CodedOutputStream.newInstance(new byte[notEnoughBytes]);
@@ -567,25 +593,25 @@ public class CodedOutputStreamTest extends TestCase {
     String invalidString = newString(Character.MIN_HIGH_SURROGATE, 'f', 'o', 'o', 'b', 'a', 'r');
     try {
       outputWithArray.writeStringNoTag(invalidString);
-      fail("Expected OutOfSpaceException");
+      assertWithMessage("Expected OutOfSpaceException").fail();
     } catch (OutOfSpaceException e) {
-      assertTrue(e.getCause() instanceof IndexOutOfBoundsException);
+      assertThat(e).hasCauseThat().isInstanceOf(IndexOutOfBoundsException.class);
     }
     try {
       outputWithByteBuffer.writeStringNoTag(invalidString);
-      fail("Expected OutOfSpaceException");
+      assertWithMessage("Expected OutOfSpaceException").fail();
     } catch (OutOfSpaceException e) {
-      assertTrue(e.getCause() instanceof IndexOutOfBoundsException);
+      assertThat(e).hasCauseThat().isInstanceOf(IndexOutOfBoundsException.class);
     }
   }
 
   /** Regression test for https://github.com/protocolbuffers/protobuf/issues/292 */
+  @Test
   public void testCorrectExceptionThrowWhenEncodingStringsWithoutEnoughSpace() throws Exception {
     String testCase = "Foooooooo";
-    assertEquals(
-        CodedOutputStream.computeUInt32SizeNoTag(testCase.length()),
-        CodedOutputStream.computeUInt32SizeNoTag(testCase.length() * 3));
-    assertEquals(11, CodedOutputStream.computeStringSize(1, testCase));
+    assertThat(CodedOutputStream.computeUInt32SizeNoTag(testCase.length()))
+        .isEqualTo(CodedOutputStream.computeUInt32SizeNoTag(testCase.length() * 3));
+    assertThat(CodedOutputStream.computeStringSize(1, testCase)).isEqualTo(11);
     // Tag is one byte, varint describing string length is 1 byte, string length is 9 bytes.
     // An array of size 1 will cause a failure when trying to write the varint.
     for (OutputType outputType :
@@ -599,13 +625,14 @@ public class CodedOutputStreamTest extends TestCase {
         Coder coder = outputType.newCoder(i);
         try {
           coder.stream().writeString(1, testCase);
-          fail("Should have thrown an out of space exception");
+          assertWithMessage("Should have thrown an out of space exception").fail();
         } catch (CodedOutputStream.OutOfSpaceException expected) {
         }
       }
     }
   }
 
+  @Test
   public void testDifferentStringLengths() throws Exception {
     // Test string serialization roundtrip using strings of the following lengths,
     // with ASCII and Unicode characters requiring different UTF-8 byte counts per
@@ -631,6 +658,7 @@ public class CodedOutputStreamTest extends TestCase {
     }
   }
 
+  @Test
   public void testNioEncodersWithInitialOffsets() throws Exception {
     String value = "abc";
     for (Coder coder :
@@ -696,10 +724,9 @@ public class CodedOutputStreamTest extends TestCase {
     Coder coder = outputType.newCoder(testAllTypes.getSerializedSize());
     testAllTypes.writeTo(coder.stream());
     coder.stream().flush();
-    assertEquals(
-        "OuputType: " + outputType,
-        fullString,
-        TestAllTypes.parseFrom(coder.toByteArray()).getOptionalString());
+    assertWithMessage("OuputType: " + outputType)
+        .that(fullString)
+        .isEqualTo(TestAllTypes.parseFrom(coder.toByteArray()).getOptionalString());
   }
 
   private static String fullString(char c, int length) {
@@ -730,13 +757,14 @@ public class CodedOutputStreamTest extends TestCase {
   }
 
   private static void assertEqualBytes(OutputType outputType, byte[] a, byte[] b) {
-    assertEquals(outputType.name(), toList(a), toList(b));
+    assertWithMessage(outputType.name()).that(toList(a)).isEqualTo(toList(b));
   }
 
   /**
    * Writes the given value using writeRawVarint32() and writeRawVarint64() and checks that the
    * result matches the given bytes.
    */
+  @SuppressWarnings("UnnecessaryLongToIntConversion") // Intentionally tests 32-bit int values.
   private static void assertWriteVarint(byte[] data, long value) throws Exception {
     for (OutputType outputType : OutputType.values()) {
       // Only test 32-bit write if the value fits into an int.
@@ -747,7 +775,7 @@ public class CodedOutputStreamTest extends TestCase {
         assertEqualBytes(outputType, data, coder.toByteArray());
 
         // Also try computing size.
-        assertEquals(data.length, CodedOutputStream.computeUInt32SizeNoTag((int) value));
+        assertThat(data).hasLength(CodedOutputStream.computeUInt32SizeNoTag((int) value));
       }
 
       {
@@ -757,7 +785,7 @@ public class CodedOutputStreamTest extends TestCase {
         assertEqualBytes(outputType, data, coder.toByteArray());
 
         // Also try computing size.
-        assertEquals(data.length, CodedOutputStream.computeUInt64SizeNoTag(value));
+        assertThat(data).hasLength(CodedOutputStream.computeUInt64SizeNoTag(value));
       }
     }
 
@@ -792,10 +820,11 @@ public class CodedOutputStreamTest extends TestCase {
       coder.stream().writeUInt64NoTag(value);
       coder.stream().flush();
       byte[] bytes = coder.toByteArray();
-      assertEquals(
-          outputType.name(), bytes.length, CodedOutputStream.computeUInt64SizeNoTag(value));
+      assertWithMessage(outputType.name())
+          .that(bytes)
+          .hasLength(CodedOutputStream.computeUInt64SizeNoTag(value));
       CodedInputStream input = CodedInputStream.newInstance(new ByteArrayInputStream(bytes));
-      assertEquals(outputType.name(), value, input.readRawVarint64());
+      assertWithMessage(outputType.name()).that(input.readRawVarint64()).isEqualTo(value);
     }
 
     if (value == (int) value) {
@@ -803,10 +832,11 @@ public class CodedOutputStreamTest extends TestCase {
       coder.stream().writeUInt32NoTag((int) value);
       coder.stream().flush();
       byte[] bytes = coder.toByteArray();
-      assertEquals(
-          outputType.name(), bytes.length, CodedOutputStream.computeUInt32SizeNoTag((int) value));
+      assertWithMessage(outputType.name())
+          .that(bytes)
+          .hasLength(CodedOutputStream.computeUInt32SizeNoTag((int) value));
       CodedInputStream input = CodedInputStream.newInstance(new ByteArrayInputStream(bytes));
-      assertEquals(outputType.name(), value, input.readRawVarint32());
+      assertWithMessage(outputType.name()).that(input.readRawVarint32()).isEqualTo(value);
     }
   }
 }
